package djmil.cordacheckers.gameproposal;

import java.net.URI;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.util.UriComponentsBuilder;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.GameState;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.Stone;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.ReqGameProposalCreate;
import djmil.cordacheckers.user.HoldingIdentityResolver;
import djmil.cordacheckers.user.User;


@RestController
@RequestMapping("gameproposal")
public class GameProposalController {

    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @GetMapping
    public ResponseEntity<String> findAllUnconsumed(
        @AuthenticationPrincipal User player
    ) {
        List<GameState> gsList = cordaClient.gameStateList(player.getHoldingIdentity());

        return ResponseEntity.ok(gsList.toString());
    }

    // @PostMapping()
    // public  ResponseEntity<String> gameproposalSend(@AuthenticationPrincipal ApiUserDetails user) {
        
    //     return  ResponseEntity.ok("");
    // }
    
    @PostMapping()
    public ResponseEntity<Void> createGameProposal(
        @AuthenticationPrincipal User sender,
        @RequestBody ReqGameProposalCreate gpRequest,
        UriComponentsBuilder ucb
    ) throws JsonMappingException, JsonProcessingException {
        final HoldingIdentity gpSender = sender.getHoldingIdentity();
        // TODO: throw execption with custom type
        final HoldingIdentity gpReceiver = holdingIdentityResolver.getByUsername(gpRequest.opponentName());
        final Stone.Color gpReceiverColor = gpRequest.opponentColor();

        // TODO handle expectionns here
        GameState gameStateView = cordaClient.gameProposalCreate(
            gpSender,
            gpReceiver,
            gpReceiverColor,
            gpRequest.message()
        );

        URI locationOfNewGameProposal = ucb
            .path("gameproposal/{id}")
            .buildAndExpand(gameStateView)
            .toUri();

        return ResponseEntity
            .created(locationOfNewGameProposal)
            .build();
    }

}