package djmil.cordacheckers.gameresult;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.VNode;
import djmil.cordacheckers.states.GameResultState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.ledger.utxo.UtxoLedgerService;

public class RankingFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(RankingFlow.class);

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @CordaInject
    public MemberLookup memberLookup;

    @Suspendable
    @Override
    public String call(ClientRequestBody requestBody) {
        try {
            final List<GameResultState> gameStateResutList = getGameResultsList();

            Map<String, Rank> leaderboard = newLeaderboard();
            for (GameResultState gs : gameStateResutList) {
             
                boolean isDraw = true;
                if (gs.getWinnerName() != null) {
                    final var winnerCommonName =  gs.getWinnerName().getCommonName();
                    leaderboard.put(winnerCommonName, leaderboard.get(winnerCommonName).gameWon() );
                    isDraw = false;
                }

                final var blackPlayer = gs.getBlackPlayer().getCommonName();
                leaderboard.put(blackPlayer, leaderboard.get(blackPlayer).gamePlayed().gameDraw(isDraw) );

                final var whitePlayer = gs.getWhitePlayer().getCommonName();
                leaderboard.put(whitePlayer, leaderboard.get(whitePlayer).gamePlayed().gameDraw(isDraw) );
            }

            return new RankingFlowResponce(leaderboard)
                .toJsonEncodedString(jsonMarshallingService);
        } catch (Exception e) {
            log.warn(requestBody + " [ERROR] " +e.toString());
            return new RankingFlowResponce(e)
                .toJsonEncodedString(jsonMarshallingService);
        }
    }

    @Suspendable
    List<GameResultState> getGameResultsList() {
        return utxoLedgerService
            .findUnconsumedStatesByType(GameResultState.class)
            .stream()
            .map( stateAndRef -> stateAndRef.getState().getContractState() )
            .collect(Collectors.toList());
    }

    @Suspendable
    Map<String, Rank> newLeaderboard() {
        Map<String, Rank> leaderboard = new HashMap<>();

        memberLookup.lookup()
            .stream()
            .filter(member -> VNode.isCordaCherckersPlayer(member) )
            .forEach(member -> leaderboard.put(member.getName().getCommonName(), new Rank()) );

        return leaderboard;
    }

}
